<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>OCR Correction Tool</title>
    <link rel="stylesheet" href="css/style.css">
</head>
<body>
    <header>
        <h1>OCR Correction Tool</h1>
    </header>

    <main id="app">
        <p>Loading application...</p>
    </main>

    <footer>
        <p>Generated by natural-pdf</p>
    </footer>

    <script src="js/lib/react.development.js"></script>
    <script src="js/lib/react-dom.development.js"></script>
    <script src="js/lib/babel.min.js"></script>
    <script src="js/lib/jszip.min.js"></script>
    <script src="js/lib/FileSaver.min.js"></script>

    <script type="text/babel" src="js/app.js"></script>

</body>
</html> 